@@ -124,6 +124,14 @@ class union_map:
             raise
         res = isl.isl_union_map_gist_domain(isl.isl_union_map_copy(self.ptr), isl.isl_union_set_copy(arg1.ptr))
         return union_map(ctx=self.ctx, ptr=res)
+    def gist_range(self, arg1):
+        try:
+            if not arg1.__class__ is union_set:
+                arg1 = union_set(arg1)
+        except:
+            raise
+        res = isl.isl_union_map_gist_range(isl.isl_union_map_copy(self.ptr), isl.isl_union_set_copy(arg1.ptr))
+        return union_map(ctx=self.ctx, ptr=res)
     def intersect_domain(self, arg1):
         try:
             if not arg1.__class__ is union_set:
@@ -235,6 +243,7 @@ isl.isl_union_map_intersect_params.restype = c_void_p
 isl.isl_union_map_gist.restype = c_void_p
 isl.isl_union_map_gist_params.restype = c_void_p
 isl.isl_union_map_gist_domain.restype = c_void_p
+isl.isl_union_map_gist_range.restype = c_void_p
 isl.isl_union_map_intersect_domain.restype = c_void_p
 isl.isl_union_map_intersect_range.restype = c_void_p
 isl.isl_union_map_apply_domain.restype = c_void_p
@@ -344,6 +353,9 @@ class map(union_map):
             return union_map(self).subtract(arg1)
         res = isl.isl_map_subtract(isl.isl_map_copy(self.ptr), isl.isl_map_copy(arg1.ptr))
         return map(ctx=self.ctx, ptr=res)
+    def complement(self):
+        res = isl.isl_map_complement(isl.isl_map_copy(self.ptr))
+        return map(ctx=self.ctx, ptr=res)
     def deltas(self):
         res = isl.isl_map_deltas(isl.isl_map_copy(self.ptr))
         return set(ctx=self.ctx, ptr=res)
@@ -456,6 +468,7 @@ isl.isl_map_apply_range.restype = c_void_p
 isl.isl_map_intersect.restype = c_void_p
 isl.isl_map_intersect_params.restype = c_void_p
 isl.isl_map_subtract.restype = c_void_p
+isl.isl_map_complement.restype = c_void_p
 isl.isl_map_deltas.restype = c_void_p
 isl.isl_map_detect_equalities.restype = c_void_p
 isl.isl_map_affine_hull.restype = c_void_p
@@ -569,6 +582,14 @@ class basic_map(map):
     def is_empty(self):
         res = isl.isl_basic_map_is_empty(self.ptr)
         return res
+    def is_subset(self, arg1):
+        try:
+            if not arg1.__class__ is basic_map:
+                arg1 = basic_map(arg1)
+        except:
+            return map(self).is_subset(arg1)
+        res = isl.isl_basic_map_is_subset(self.ptr, arg1.ptr)
+        return res
     def deltas(self):
         res = isl.isl_basic_map_deltas(isl.isl_basic_map_copy(self.ptr))
         return basic_set(ctx=self.ctx, ptr=res)
@@ -779,14 +800,14 @@ class set(union_set):
             self.ctx = keywords["ctx"]
             self.ptr = keywords["ptr"]
             return
-        if len(args) == 1 and type(args[0]) == str:
-            self.ctx = Context.getDefaultInstance()
-            self.ptr = isl.isl_set_read_from_str(self.ctx, args[0])
-            return
         if len(args) == 1 and args[0].__class__ is basic_set:
             self.ctx = Context.getDefaultInstance()
             self.ptr = isl.isl_set_from_basic_set(isl.isl_basic_set_copy(args[0].ptr))
             return
+        if len(args) == 1 and type(args[0]) == str:
+            self.ctx = Context.getDefaultInstance()
+            self.ptr = isl.isl_set_read_from_str(self.ctx, args[0])
+            return
         raise Error
     def __del__(self):
         if hasattr(self, 'ptr'):
@@ -798,40 +819,9 @@ class set(union_set):
         return res
     def __repr__(self):
         return 'isl.set("%s")' % str(self)
-    def gist(self, arg1):
-        try:
-            if not arg1.__class__ is set:
-                arg1 = set(arg1)
-        except:
-            return union_set(self).gist(arg1)
-        res = isl.isl_set_gist(isl.isl_set_copy(self.ptr), isl.isl_set_copy(arg1.ptr))
-        return set(ctx=self.ctx, ptr=res)
-    def coalesce(self):
-        res = isl.isl_set_coalesce(isl.isl_set_copy(self.ptr))
-        return set(ctx=self.ctx, ptr=res)
-    def foreach_basic_set(self, arg1):
-        exc_info = [None]
-        fn = CFUNCTYPE(c_int, c_void_p, c_void_p)
-        def cb_func(cb_arg0, cb_arg1):
-            cb_arg0 = basic_set(ctx=self.ctx, ptr=cb_arg0)
-            try:
-                arg1(cb_arg0)
-            except:
-                import sys
-                exc_info[0] = sys.exc_info()
-                return -1
-            return 0
-        cb = fn(cb_func)
-        res = isl.isl_set_foreach_basic_set(self.ptr, cb, None)
-        if exc_info[0] != None:
-            raise exc_info[0][0], exc_info[0][1], exc_info[0][2]
-        return res
-    def lexmin(self):
-        res = isl.isl_set_lexmin(isl.isl_set_copy(self.ptr))
-        return set(ctx=self.ctx, ptr=res)
-    def lexmax(self):
-        res = isl.isl_set_lexmax(isl.isl_set_copy(self.ptr))
-        return set(ctx=self.ctx, ptr=res)
+    def sample(self):
+        res = isl.isl_set_sample(isl.isl_set_copy(self.ptr))
+        return basic_set(ctx=self.ctx, ptr=res)
     def detect_equalities(self):
         res = isl.isl_set_detect_equalities(isl.isl_set_copy(self.ptr))
         return set(ctx=self.ctx, ptr=res)
@@ -911,6 +901,40 @@ class set(union_set):
             return union_set(self).is_equal(arg1)
         res = isl.isl_set_is_equal(self.ptr, arg1.ptr)
         return res
+    def lexmin(self):
+        res = isl.isl_set_lexmin(isl.isl_set_copy(self.ptr))
+        return set(ctx=self.ctx, ptr=res)
+    def lexmax(self):
+        res = isl.isl_set_lexmax(isl.isl_set_copy(self.ptr))
+        return set(ctx=self.ctx, ptr=res)
+    def gist(self, arg1):
+        try:
+            if not arg1.__class__ is set:
+                arg1 = set(arg1)
+        except:
+            return union_set(self).gist(arg1)
+        res = isl.isl_set_gist(isl.isl_set_copy(self.ptr), isl.isl_set_copy(arg1.ptr))
+        return set(ctx=self.ctx, ptr=res)
+    def coalesce(self):
+        res = isl.isl_set_coalesce(isl.isl_set_copy(self.ptr))
+        return set(ctx=self.ctx, ptr=res)
+    def foreach_basic_set(self, arg1):
+        exc_info = [None]
+        fn = CFUNCTYPE(c_int, c_void_p, c_void_p)
+        def cb_func(cb_arg0, cb_arg1):
+            cb_arg0 = basic_set(ctx=self.ctx, ptr=cb_arg0)
+            try:
+                arg1(cb_arg0)
+            except:
+                import sys
+                exc_info[0] = sys.exc_info()
+                return -1
+            return 0
+        cb = fn(cb_func)
+        res = isl.isl_set_foreach_basic_set(self.ptr, cb, None)
+        if exc_info[0] != None:
+            raise exc_info[0][0], exc_info[0][1], exc_info[0][2]
+        return res
     def identity(self):
         res = isl.isl_set_identity(isl.isl_set_copy(self.ptr))
         return map(ctx=self.ctx, ptr=res)
@@ -921,14 +945,11 @@ class set(union_set):
         res = isl.isl_set_flatten(isl.isl_set_copy(self.ptr))
         return set(ctx=self.ctx, ptr=res)
 
-isl.isl_set_read_from_str.restype = c_void_p
-isl.isl_set_read_from_str.argtypes = [Context, c_char_p]
 isl.isl_set_from_basic_set.restype = c_void_p
 isl.isl_set_from_basic_set.argtypes = [c_void_p]
-isl.isl_set_gist.restype = c_void_p
-isl.isl_set_coalesce.restype = c_void_p
-isl.isl_set_lexmin.restype = c_void_p
-isl.isl_set_lexmax.restype = c_void_p
+isl.isl_set_read_from_str.restype = c_void_p
+isl.isl_set_read_from_str.argtypes = [Context, c_char_p]
+isl.isl_set_sample.restype = c_void_p
 isl.isl_set_detect_equalities.restype = c_void_p
 isl.isl_set_affine_hull.restype = c_void_p
 isl.isl_set_polyhedral_hull.restype = c_void_p
@@ -938,6 +959,10 @@ isl.isl_set_intersect_params.restype = c_void_p
 isl.isl_set_subtract.restype = c_void_p
 isl.isl_set_complement.restype = c_void_p
 isl.isl_set_apply.restype = c_void_p
+isl.isl_set_lexmin.restype = c_void_p
+isl.isl_set_lexmax.restype = c_void_p
+isl.isl_set_gist.restype = c_void_p
+isl.isl_set_coalesce.restype = c_void_p
 isl.isl_set_identity.restype = c_void_p
 isl.isl_set_flatten.restype = c_void_p
 isl.isl_set_free.argtypes = [c_void_p]
@@ -965,14 +990,6 @@ class basic_set(set):
         return res
     def __repr__(self):
         return 'isl.basic_set("%s")' % str(self)
-    def gist(self, arg1):
-        try:
-            if not arg1.__class__ is basic_set:
-                arg1 = basic_set(arg1)
-        except:
-            return set(self).gist(arg1)
-        res = isl.isl_basic_set_gist(isl.isl_basic_set_copy(self.ptr), isl.isl_basic_set_copy(arg1.ptr))
-        return basic_set(ctx=self.ctx, ptr=res)
     def intersect(self, arg1):
         try:
             if not arg1.__class__ is basic_set:
@@ -981,6 +998,14 @@ class basic_set(set):
             return set(self).intersect(arg1)
         res = isl.isl_basic_set_intersect(isl.isl_basic_set_copy(self.ptr), isl.isl_basic_set_copy(arg1.ptr))
         return basic_set(ctx=self.ctx, ptr=res)
+    def intersect_params(self, arg1):
+        try:
+            if not arg1.__class__ is basic_set:
+                arg1 = basic_set(arg1)
+        except:
+            return set(self).intersect_params(arg1)
+        res = isl.isl_basic_set_intersect_params(isl.isl_basic_set_copy(self.ptr), isl.isl_basic_set_copy(arg1.ptr))
+        return basic_set(ctx=self.ctx, ptr=res)
     def apply(self, arg1):
         try:
             if not arg1.__class__ is basic_map:
@@ -992,6 +1017,9 @@ class basic_set(set):
     def affine_hull(self):
         res = isl.isl_basic_set_affine_hull(isl.isl_basic_set_copy(self.ptr))
         return basic_set(ctx=self.ctx, ptr=res)
+    def sample(self):
+        res = isl.isl_basic_set_sample(isl.isl_basic_set_copy(self.ptr))
+        return basic_set(ctx=self.ctx, ptr=res)
     def detect_equalities(self):
         res = isl.isl_basic_set_detect_equalities(isl.isl_basic_set_copy(self.ptr))
         return basic_set(ctx=self.ctx, ptr=res)
@@ -1015,6 +1043,22 @@ class basic_set(set):
     def is_empty(self):
         res = isl.isl_basic_set_is_empty(self.ptr)
         return res
+    def is_subset(self, arg1):
+        try:
+            if not arg1.__class__ is basic_set:
+                arg1 = basic_set(arg1)
+        except:
+            return set(self).is_subset(arg1)
+        res = isl.isl_basic_set_is_subset(self.ptr, arg1.ptr)
+        return res
+    def gist(self, arg1):
+        try:
+            if not arg1.__class__ is basic_set:
+                arg1 = basic_set(arg1)
+        except:
+            return set(self).gist(arg1)
+        res = isl.isl_basic_set_gist(isl.isl_basic_set_copy(self.ptr), isl.isl_basic_set_copy(arg1.ptr))
+        return basic_set(ctx=self.ctx, ptr=res)
     def is_wrapping(self):
         res = isl.isl_basic_set_is_wrapping(self.ptr)
         return res
@@ -1024,14 +1068,16 @@ class basic_set(set):
 
 isl.isl_basic_set_read_from_str.restype = c_void_p
 isl.isl_basic_set_read_from_str.argtypes = [Context, c_char_p]
-isl.isl_basic_set_gist.restype = c_void_p
 isl.isl_basic_set_intersect.restype = c_void_p
+isl.isl_basic_set_intersect_params.restype = c_void_p
 isl.isl_basic_set_apply.restype = c_void_p
 isl.isl_basic_set_affine_hull.restype = c_void_p
+isl.isl_basic_set_sample.restype = c_void_p
 isl.isl_basic_set_detect_equalities.restype = c_void_p
 isl.isl_basic_set_lexmin.restype = c_void_p
 isl.isl_basic_set_lexmax.restype = c_void_p
 isl.isl_basic_set_union.restype = c_void_p
+isl.isl_basic_set_gist.restype = c_void_p
 isl.isl_basic_set_flatten.restype = c_void_p
 isl.isl_basic_set_free.argtypes = [c_void_p]
 isl.isl_basic_set_to_str.argtypes = [c_void_p]
