@@ -1146,6 +1146,12 @@ int isl_basic_map_is_div_constraint(__isl_keep isl_basic_map *bmap,
 	return 1;
 }
 
+int isl_basic_set_is_div_constraint(__isl_keep isl_basic_set *bset,
+	isl_int *constraint, unsigned div)
+{
+	return isl_basic_map_is_div_constraint(bset, constraint, div);
+}
+
 
 /* If the only constraints a div d=floor(f/m)
  * appears in are its two defining constraints
@@ -1579,7 +1585,7 @@ static __isl_give isl_basic_set *uset_gist_full(__isl_take isl_basic_set *bset,
 	context_ineq = context->n_ineq;
 	combined = isl_basic_set_cow(isl_basic_set_copy(context));
 	combined = isl_basic_set_extend_constraints(combined, 0, bset->n_ineq);
-	tab = isl_tab_from_basic_set(combined);
+	tab = isl_tab_from_basic_set(combined, 0);
 	for (i = 0; i < context_ineq; ++i)
 		if (isl_tab_freeze_constraint(tab, i) < 0)
 			goto error;
@@ -1768,10 +1774,8 @@ struct isl_basic_map *isl_basic_map_gist(struct isl_basic_map *bmap,
 		return bmap;
 	}
 	if (isl_basic_map_plain_is_empty(context)) {
-		isl_space *dim = isl_space_copy(bmap->dim);
-		isl_basic_map_free(context);
 		isl_basic_map_free(bmap);
-		return isl_basic_map_universe(dim);
+		return context;
 	}
 	if (isl_basic_map_plain_is_empty(bmap)) {
 		isl_basic_map_free(context);
@@ -1809,10 +1813,8 @@ __isl_give isl_map *isl_map_gist_basic_map(__isl_take isl_map *map,
 		goto error;;
 
 	if (isl_basic_map_plain_is_empty(context)) {
-		isl_space *dim = isl_space_copy(map->dim);
-		isl_basic_map_free(context);
 		isl_map_free(map);
-		return isl_map_universe(dim);
+		return isl_map_from_basic_map(context);
 	}
 
 	context = isl_basic_map_remove_redundancies(context);
@@ -1871,6 +1873,15 @@ __isl_give isl_set *isl_set_gist_basic_set(__isl_take isl_set *set,
 					(struct isl_basic_map *)context);
 }
 
+__isl_give isl_set *isl_set_gist_params_basic_set(__isl_take isl_set *set,
+	__isl_take isl_basic_set *context)
+{
+	isl_space *space = isl_set_get_space(set);
+	isl_basic_set *dom_context = isl_basic_set_universe(space);
+	dom_context = isl_basic_set_intersect_params(dom_context, context);
+	return isl_set_gist_basic_set(set, dom_context);
+}
+
 __isl_give isl_set *isl_set_gist(__isl_take isl_set *set,
 	__isl_take isl_set *context)
 {
@@ -1886,6 +1897,14 @@ __isl_give isl_map *isl_map_gist_domain(__isl_take isl_map *map,
 	return isl_map_gist(map, map_context);
 }
 
+__isl_give isl_map *isl_map_gist_range(__isl_take isl_map *map,
+	__isl_take isl_set *context)
+{
+	isl_map *map_context = isl_map_universe(isl_map_get_space(map));
+	map_context = isl_map_intersect_range(map_context, context);
+	return isl_map_gist(map, map_context);
+}
+
 __isl_give isl_map *isl_map_gist_params(__isl_take isl_map *map,
 	__isl_take isl_set *context)
 {
@@ -2194,7 +2213,7 @@ static struct isl_basic_map *drop_more_redundant_divs(
 	if (!vec)
 		goto error;
 
-	tab = isl_tab_from_basic_map(bmap);
+	tab = isl_tab_from_basic_map(bmap, 0);
 
 	while (n > 0) {
 		int i, l, u;
